function [samples, indicators] = sampleGMM(mus, sigmas, weights, nSamples)
% Generates samples from a Gaussian mixture model.
%
% [SAMPLES, INDICATOR] = sampleGMM(MUS, SIGMAS, WEIGHTS, NSAMPLES) creates
% a matrix SAMPLES containing a number NSAMPLES of random vectors from a
% Gaussian mixture model that is defined by the mean vectors in MUS, the
% covariance matrices in SIGMAS, and the relative mixture weights in
% WEIGHTS. The vector INDICATORS stores the mixture assignments.
%
% Input
% -----
% mus: 
% An (nComp x nDims) matrix containing the mean values of the mixture
% components. "nDims" is the dimensionality of the sample space and "nComp"
% is the number of mixture components.
%
% sigmas:  
% An (nDims x nDims x nComp) array containing the covariance
% matrices of the mixture components.
%
% weights:
% An (nComp x 1) or (1 x nComp) vector containing the mixing weights that
% sum to 1.
%
% nSamples:
% An integer specifying the number of samples to be generated.
%
%
% Output
% ------
% samples:
% An (nSamples x nDims) matrix containing the generated samples.
%
% indicators:
% An (nSamples x 1) vector of integers in {1,...,nComp} indicating the
% mixture assignments of all samples.
%
% See also gmdistribution


%%% dimension of the sample space and number of components %%%
[nComp, nDims] = size(mus);

%%% sample indicators %%%
indicators = randsample(1:nComp, nSamples, true, weights)';

%%% container variable to store the samples %%%
samples = zeros(nSamples, nDims);

%%% sample component-wise %%%
for c = 1:nComp
    %%% indices belonging to current component %%% 
    inds = indicators == c;
    
    %%% generate the samples %%%
    samples(inds,:) = mvnrnd(mus(c,:), sigmas(:,:,c), sum(inds));
end



